#!/usr/bin/env python
# -*- Mode: python; coding: utf-8; tab-width: 8; indent-tabs-mode: t; -*-
#
# Copyright (C) 2007 - Daniel Bessler <danielb@kiba-dock.org>
# Note: based on awns firefox plugin

import wnck
import gobject
import gtk
import time
from sys import exit
from commands import getoutput

import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gobject

PLUGIN_NAME = "Kiba-Firefox"

if getoutput("kiba-settings --get-bool firefox_enable") != "TRUE":
	exit()

class Firefox:
	icon_added=0
	running=0
		
	def __init__ (self):
		#start timeout, 1000 means every second
		source_id=gobject.timeout_add(1000, self.timeout_func)
		
		#get the bus proxy
		self.bus = dbus.SessionBus()
		
		#Setup Signal Handlers (we need to avoid some uggly segfaults when kiba closes/reopenes)
		self.bus.add_signal_receiver(self.open_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "Initialled")
		self.bus.add_signal_receiver(self.close_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "Closed")
		self.bus.add_signal_receiver(self.context_menu_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "IconMenu")
		self.bus.add_signal_receiver(self.button_release_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "IconReleased")
		
		#check if kiba is running on startup
		if getoutput("pidof kiba-dock") != "":
			self.running=1
	
	def add_kiba_icon(self):
		#Initiall Kiba-DBus stuff
		self.obj = self.bus.get_object("org.kiba.dock.Kiba", "/org/kiba/dock/Kiba")
		self.kiba = dbus.Interface(self.obj, "org.kiba.dock.Kiba")
		
		#Add Icon to Kiba
		self.kiba.AddObject(PLUGIN_NAME, "firefox-icon")
		self.kiba.NewProgress (PLUGIN_NAME, 100);
		
	def context_menu_handler(self, id):
		if id != PLUGIN_NAME:
			return
	
	def button_release_handler(self, id):
		if id != PLUGIN_NAME:
			return
		
		#FIXME: control just the download window (ChangeWinStateByPid)
		self.kiba.ChangeWinStateByClass(PLUGIN_NAME, "Firefox-bin")
	
	def close_handler(self, sender=None):
		self.running=0
		self.icon_added=0
	
	def open_handler(self, sender=None):
		self.running=1
	
	def timeout_func(self):
		if self.running == 0:
			return 1
		
		#get the Window name of the download window
		window_found=0
		screen = wnck.screen_get_default()
		while gtk.events_pending(): gtk.main_iteration()
		windowlist = screen.get_windows()
		names = []
		for window in windowlist:
			names.append(window.get_name())
		for name in names:
			if "Downloads" in name:
				if "%" in name:
					if self.icon_added == 0:
						self.add_kiba_icon();
						self.icon_added = 1
					percent = int(name.split("%")[0])
					if percent < 0:
						percent = 0
					self.kiba.ProgressSetValue (PLUGIN_NAME, percent)
					self.kiba.SetTitel (PLUGIN_NAME, name)
					#self.kiba.ProgressSetMessage (PLUGIN_NAME, name)
				elif self.icon_added == 1 and self.running == 1:
					self.kiba.ProgressSetValue (PLUGIN_NAME, 100);
					self.kiba.RemoveObject (PLUGIN_NAME);
					self.icon_added = 0
				window_found=1
		if self.icon_added == 1 and window_found == 0:
			self.kiba.RemoveObject (PLUGIN_NAME);
			self.icon_added = 0
		
		return 1

#We need to use the Mainloop to catch the DBus Signals
DBusGMainLoop(set_as_default=True)
loop=gobject.MainLoop()

#Load the Plugin
kiba_firefox=Firefox()

#enter mainloop
loop.run()

if getoutput("pidof kiba-dock") != "":
	kiba_firefox.kiba.RemoveObject(PLUGIN_NAME)
