#!/usr/bin/env python
# -*- Mode: python; coding: utf-8; tab-width: 8; indent-tabs-mode: t; -*-
#
# Copyright (C) 2007 - Daniel Bessler <danielb@kiba-dock.org>
# Note: based on awns amarok plugin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

#FIXME: getoutput('dcop amarok ...) uses much cpu here

from commands import getoutput
import signal
from time import sleep
from os import system
import gtk

import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gobject

PLUGIN_NAME = "kiba-amarok"

class KibaAmarokPlugin(object):
	current_cover = "-"
	old_total=0
	icon_added=0
	running=0
	source_id=0
	menu=0
	
	def __init__ (self):
		#start timeout, 1000 means every second
		source_id=gobject.timeout_add(1000, self.timeout_func)
		
		#get the bus proxy
		self.bus = dbus.SessionBus()
		
		#Setup Signal Handlers (we need to avoid some uggly segfaults when kiba closes/reopenes)
		self.bus.add_signal_receiver(self.open_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "Initialled")
		self.bus.add_signal_receiver(self.close_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "Closed")
		self.bus.add_signal_receiver(self.context_menu_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "IconMenu")
		self.bus.add_signal_receiver(self.button_release_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "IconReleased")
		
		signal.signal(signal.SIGTERM, self.cleanup)
		
		#check if kiba is running on startup
		if getoutput("pidof kiba-dock") != "":
			self.running=1
	
	def cleanup(self, signum, frame):
		if self.running == 1:
			self.kiba.RemoveObject (PLUGIN_NAME);
	
	def update_icon(self):
		if getoutput('dcop amarok player status')=='1':
			return
		elif getoutput('dcop amarok player isPlaying') == "false":
			if self.icon_added == 1:
				self.old_total=0
				self.kiba.RemoveObject (PLUGIN_NAME);
				self.icon_added = 0
			return
		if self.icon_added == 0:
			self.add_kiba_icon()
			self.icon_added = 1
		
		total = int(getoutput('dcop amarok player trackTotalTime 2> /dev/null'))
		if total != self.old_total:
			self.old_total = total;
		if total > 0:
			current = int(getoutput('dcop amarok player trackCurrentTime 2> /dev/null'))
			self.kiba.ProgressSetValue (PLUGIN_NAME, 100*current/total)
				
			cover = getoutput('dcop amarok player coverImage 2> /dev/null')
			if self.current_cover != cover or self.current_cover == "" or self.current_cover == "-":
				self.current_cover = cover
				self.kiba.SetIcon (PLUGIN_NAME, cover);
				
			title = getoutput('dcop amarok player title 2> /dev/null')
			self.kiba.SetTitel(PLUGIN_NAME, title);
			#self.kiba.BubbleSetMessage(PLUGIN_NAME, title);
	
	def add_kiba_icon(self):
		#Initiall Kiba-DBus stuff
		self.obj = self.bus.get_object("org.kiba.dock.Kiba", "/org/kiba/dock/Kiba")
		self.kiba = dbus.Interface(self.obj, "org.kiba.dock.Kiba")
		
		#Add Icon to Kiba
		self.kiba.AddObject(PLUGIN_NAME, "amarokapp", "amarok")
		
	def pause_amarok(self, *args):
		system("dcop amarok player pause")
	
	def prev_track(self, *args):
		system("dcop amarok player prev")
	
	def next_track(self, *args):
		system("dcop amarok player next")
	
	def exit_amarok(self, *args):
		system("killall amarok amarokapp")
		
	def create_menu(self):
		menu_items = (
			(gtk.STOCK_MEDIA_PREVIOUS, self.prev_track ),
			(gtk.STOCK_MEDIA_PAUSE, self.pause_amarok ),
			(gtk.STOCK_MEDIA_NEXT, self.next_track ),
			None,
			(gtk.STOCK_QUIT, self.exit_amarok),
		);
		self.menu = gtk.Menu()
		for i in menu_items:
			if i is None:
				self.menu.add(gtk.SeparatorMenuItem());
			else:
				menu_items = gtk.ImageMenuItem( i[0] );
				menu_items.connect( 'activate', i[1] );
				self.menu.add( menu_items );
				
		self.menu.show_all()
	
	def show_menu(self):
		if self.menu == 0:
			self.create_menu()
		
		self.menu.popup(None, None, None, 3, 0)
		
		return 0
	
	def context_menu_handler(self, id):
		if id != PLUGIN_NAME:
			return
		
		gobject.timeout_add(250, self.show_menu);
	
	def button_release_handler(self, id):
		if id != PLUGIN_NAME:
			return
		
		system("dcop amarok playlist togglePlaylist")
		#system("dcop amarok player mute")
		
	def close_handler(self, sender=None):
		self.old_total=0
		self.current_cover=""
		self.running=0
		self.icon_added=0
	
	def open_handler(self, sender=None):
		self.running=1
	
	def timeout_func(self):
		if self.running == 0:
			return 1
		if getoutput("pidof amarok") == "" and getoutput("pidof amarokapp") == "":
			return 1
		
		self.update_icon()
		
		return 1

#We need to use the Mainloop to catch the DBus Signals
DBusGMainLoop(set_as_default=True)
loop=gobject.MainLoop()

#Load the Plugin
kiba_amarok=KibaAmarokPlugin()

#enter mainloop
loop.run()

if getoutput("pidof kiba-dock") != "":
	kiba_amarok.kiba.RemoveObject(PLUGIN_NAME)
